<?php
/**
 * SolutionrDEVs Application
 * Rich Discord Integration
 *
 * @brief       Login handler
 * @author      Dawid Baruch <dawid.baruch@solutiondevs.pl> <a href='https://www.solutiondevs.pl'>SolutionDEVs</a>
 * @copyright   (c) 2005 - 2019 SolutionDEVs
 * @package     SolutionDEVs Apps
 * @subpackage  Rich Discord Integration
 * @link        https://www.solutiondevs.pl
 * @link        https://www.ipsbeyond.pl
 * @since       05.02.2019
 * @version     1.0.10
 */

namespace IPS\richdiscord\Login\Handler\OAuth2;

/**
 * Discord Class
 *
 * @mixin \IPS\richdiscord\Login\Handler\OAuth2\Discord
 */
class _Discord extends \IPS\Login\Handler\OAuth2
{

    public    $apiDomain       = 'https://discordapp.com/api/v6';

    protected $_cachedUserData = array();

    /**
     * Get title
     *
     * @return string
     */
    public static function getTitle()
    {
        return 'richdiscord_login_handler_discord';
    }

    /**
     * Get logo for button
     *
     * @return \IPS\Http\Url|NULL
     */
    public function logoForUcp()
    {
        return \IPS\Theme::i()->resource( 'discord.png', 'richdiscord', 'global' );
    }

    /**
     * ACP Settings Form
     *
     * @return array List of settings to save - settings will be stored to core_login_methods.login_settings DB field
     * @code
     * return array( 'savekey'    => new \IPS\Helpers\Form\[Type]( ... ), ... );
     * @endcode
     */
    public function acpForm()
    {
        $return[]                 = 'login_handler_oauth_ui';
        $return[ 'button_color' ] = new \IPS\Helpers\Form\Color( 'oauth_custom_button_color', isset( $this->settings[ 'button_color' ] ) ? $this->settings[ 'button_color' ] : '#23282d', null, array(), null, null, null, 'button_color' );
        $return[ 'button_text' ]  = new \IPS\Helpers\Form\Translatable( 'oauth_custom_button_text', null, null, array( 'placeholder' => \IPS\Member::loggedIn()->language()->addToStack( 'oauth_custom_button_text_custom_placeholder' ), 'app' => 'core', 'key' => ( $this->id ? "core_custom_oauth_{$this->id}" : null ) ), null, null, null, 'button_text' );

        return array_merge( $return, parent::acpForm() );
    }

    /**
     * Save Handler Settings
     *
     * @param array $values Values from form
     * @return array
     */
    public function acpFormSave( &$values )
    {
        $return = parent::acpFormSave( $values );

        return $return;
    }

    /**
     * [Node] Format form values from add/edit form for save
     *
     * @param array $values Values from the form
     * @return array
     */
    public function formatFormValues( $values )
    {
        $parent = parent::formatFormValues( $values );

        if( isset( $values[ 'oauth_custom_button_text' ] ) )
        {
            if( !$this->id )
            {
                $this->save();
            }
            \IPS\Lang::saveCustom( 'core', "core_custom_oauth_{$this->id}", $values[ 'oauth_custom_button_text' ] );
            unset( $values[ 'button_text' ] );
        }

        return $parent;
    }

    /**
     * Get the button color
     *
     * @return string
     */
    public function buttonColor()
    {
        return $this->settings[ 'button_color' ];
    }

    /**
     * Get the button icon
     *
     * @return string
     */
    public function buttonIcon()
    {
        $item            = \IPS\Theme::i()->resource( 'discord.png', 'richdiscord', 'global' );
        $fakeObject      = new \StdClass;
        $fakeObject->url = $item;

        return $fakeObject;
    }

    public function logoForDeviceInformation()
    {
        return ( isset( $this->settings[ 'button_icon' ] ) and $this->settings[ 'button_icon' ] ) ? \IPS\File::get( 'core_Login', $this->settings[ 'button_icon' ] ) : 'discord';
    }

    /**
     * Get button text
     *
     * @return string
     */
    public function buttonText()
    {
        return \IPS\Member::loggedIn()->language()->addToStack( 'richdiscord_title' );
    }

    /**
     * Grant Type
     *
     * @return string
     */
    protected function grantType()
    {
        return 'authorization_code';
    }

    /**
     * Authorization Endpoint
     *
     * @param \IPS\Login $login The login object
     * @return \IPS\Http\Url
     */
    protected function authorizationEndpoint( \IPS\Login $login )
    {
        return \IPS\Http\Url::external( $this->apiDomain . '/oauth2/authorize' )->setQueryString( 'type', 'web_server' );
    }

    /**
     * Token Endpoint
     *
     * @return \IPS\Http\Url
     */
    protected function tokenEndpoint()
    {
        return \IPS\Http\Url::external( $this->apiDomain . '/oauth2/token' );
    }

    /**
     * Scopes
     *
     * @param null $additional
     * @return array
     */
    protected function scopesToRequest( $additional = null )
    {
        return array( 'email', 'identify' );
    }

    /**
     * Get authenticated user's identifier (may not be a number)
     *
     * @param string $accessToken Access Token
     * @return string
     */
    protected function authenticatedUserId( $accessToken )
    {
        return $this->_userData( $accessToken )[ 'id' ];
    }

    /**
     * Get user data
     *
     * @param string $accessToken Access Token
     * @return array
     * @throws \IPS\Login\Exception The token is invalid and the user needs to reauthenticate
     * @throws \RuntimeException Unexpected error from service
     */
    protected function _userData( $accessToken )
    {

        if( !isset( $this->_cachedUserData[ $accessToken ] ) )
        {
            $response = \IPS\Http\Url::external( $this->apiDomain . "/users/@me" )
                                     ->request()
                                     ->setHeaders(
                                         array(
                                             'Authorization' => "Bearer {$accessToken}",
                                         )
                                     )
                                     ->get()
                                     ->decodeJson();

            if( isset( $response[ 'errorCode' ] ) )
            {
                throw new \IPS\Login\Exception( $response[ 'message' ], \IPS\Login\Exception::INTERNAL_ERROR );
            }

            $this->_cachedUserData[ $accessToken ] = $response;
        }

        return $this->_cachedUserData[ $accessToken ];
    }

    /**
     * Get authenticated user's username
     * May return NULL if server doesn't support this
     *
     * @param string $accessToken Access Token
     * @return string|NULL
     * @throws \Exception
     */
    protected function authenticatedUserName( $accessToken )
    {
        $data = $this->_userData( $accessToken );

        if( isset( $data[ 'username' ] ) )
        {
            return $data[ 'username' ];
        }
        elseif( isset( $data[ 'name' ] ) )
        {
            return $data[ 'name' ];
        }

        throw new \Exception;
    }

    /**
     * Get authenticated user's email address
     * May return NULL if server doesn't support this
     *
     * @param string $accessToken Access Token
     * @return string|NULL
     */
    protected function authenticatedEmail( $accessToken )
    {
        return $this->_userData( $accessToken )[ 'email' ];
    }
}

